@@ -31,6 +31,7 @@ rb_ldap_conn_free (RB_LDAP_DATA * ldapdata)
     {
       ldap_unbind (ldapdata->ldap);
     };
+  xfree(ldapdata);
 };
 
 static void
@@ -167,6 +168,26 @@ rb_ldap_conn_s_open (int argc, VALUE argv[], VALUE klass)
   return conn;
 };
 
+/*
+ * call-seq:
+ * LDAP::Conn.open_uri(uri)  => LDAP::Conn
+ *
+ * Return a new LDAP::Conn connection to the server described with +uri+.
+ */
+VALUE
+rb_ldap_conn_s_open_uri (VALUE klass, VALUE uri)
+{
+  LDAP  *cldap = NULL;
+  int    rc;
+
+  rc = ldap_initialize (&cldap, StringValueCStr (uri));
+
+  if (rc || cldap == NULL)
+    rb_raise (rb_eLDAP_ResultError, "can't open an LDAP session");
+  
+  return rb_ldap_conn_new (klass, cldap);
+};
+
 /*
  * call-seq:
  * conn.start_tls  => nil
@@ -450,6 +471,15 @@ rb_ldap_conn_set_option (VALUE self, VALUE opt, VALUE data)
 
   switch (copt)
     {
+#if defined(USE_OPENLDAP1) || defined(USE_OPENLDAP2)
+    case LDAP_OPT_NETWORK_TIMEOUT:
+      {
+        struct timeval tv;
+        tv = rb_time_interval(data);
+        optdata = &tv;
+      }
+      break;
+#endif
     case LDAP_OPT_REFERRALS:
       optdata = (void *) NUM2INT (data);
       break;
@@ -574,6 +604,20 @@ rb_ldap_conn_get_option (VALUE self, VALUE opt)
       ldapdata->err = ldap_get_option (NULL, copt, (void *) info);
       data = (long *) info;
     }
+#if defined(USE_OPENLDAP1) || defined(USE_OPENLDAP2)
+  else if (copt == LDAP_OPT_NETWORK_TIMEOUT)
+    {
+      struct timeval tv;
+      ldapdata->err = ldap_get_option (ldapdata->ldap, copt, &tv);
+      if (!tv.tv_sec)
+        {
+          long x = -1;
+          data = &x;
+        }
+      else
+        data = (void *) tv.tv_sec;
+    }
+#endif
   else
     {
       data = (void *) ALLOCA_N (char, LDAP_GET_OPT_MAX_BUFFER_SIZE);
@@ -595,6 +639,9 @@ rb_ldap_conn_get_option (VALUE self, VALUE opt)
 	case LDAP_OPT_RESTART:
 	case LDAP_OPT_PROTOCOL_VERSION:
 	case LDAP_OPT_ERROR_NUMBER:
+#if defined(USE_OPENLDAP1) || defined(USE_OPENLDAP2)
+	case LDAP_OPT_NETWORK_TIMEOUT:
+#endif
 #ifdef USE_OPENLDAP2
 #ifdef LDAP_OPT_X_TLS
 	case LDAP_OPT_X_TLS:
@@ -653,7 +700,7 @@ rb_ldap_conn_get_option (VALUE self, VALUE opt)
     }
   else
     {
-      rb_raise (rb_eLDAP_Error, ldap_err2string (ldapdata->err));
+      rb_raise (rb_eLDAP_Error, "%s", ldap_err2string (ldapdata->err));
     };
 #else
   rb_notimplement ();
@@ -742,7 +789,7 @@ rb_ldap_conn_get_errno (VALUE self)
   GET_LDAP_DATA (self, ldapdata);
 
 #ifdef USE_NETSCAPE_SDK
-  cerr = ldap_get_lderrno (ldapdata->ldap, NULL, NULL);
+  int cerr = ldap_get_lderrno (ldapdata->ldap, NULL, NULL);
   err = INT2NUM (cerr);
 #else
 # ifdef USE_OPENLDAP1
@@ -1515,6 +1562,7 @@ rb_ldap_conn_modify_s (VALUE self, VALUE dn, VALUE attrs)
 
   ldapdata->err = ldap_modify_s (ldapdata->ldap, c_dn, c_attrs);
   Check_LDAP_Result (ldapdata->err);
+  free(c_attrs);
 
   return self;
 };
@@ -1604,6 +1652,45 @@ rb_ldap_conn_modrdn_s (VALUE self, VALUE dn, VALUE newrdn, VALUE delete_p)
   return self;
 };
 
+#if defined(HAVE_LDAPCONTROL) && defined(HAVE_LDAP_RENAME_S)
+/*
+ * call-seq:
+ * conn.rename(dn, new_rdn, new_parent_dn, delete_old_rdn, sctrls, cctrls)  => self
+ *
+ * Modify the RDN of the entry with DN, +dn+, giving it the new RDN in
+ * parent +new_parent_dn+, +new_rdn+. If you don't want to change
+ * parent, specify *nil* to +new_parent_dn+.  If +delete_old_rdn+ is
+ * *true*, the old RDN value will be deleted from the entry.
+ */
+VALUE
+rb_ldap_conn_rename_s (VALUE self, VALUE dn, VALUE newrdn, VALUE newparentdn, VALUE delete_p,
+			VALUE serverctrls, VALUE clientctrls)
+{
+  RB_LDAP_DATA *ldapdata;
+  char *c_dn;
+  char *c_newrdn;
+  char *c_newparentdn = NULL;
+  int c_delete_p;
+  LDAPControl **sctrls, **cctrls;
+
+  GET_LDAP_DATA (self, ldapdata);
+  c_dn = StringValueCStr (dn);
+  c_newrdn = StringValueCStr (newrdn);
+  if (!NIL_P(newparentdn)) {
+    c_newparentdn = StringValueCStr (newparentdn);
+  }
+  c_delete_p = (delete_p == Qtrue) ? 1 : 0;
+  sctrls = rb_ldap_get_controls (serverctrls);
+  cctrls = rb_ldap_get_controls (clientctrls);
+
+  ldapdata->err =
+    ldap_rename_s (ldapdata->ldap, c_dn, c_newrdn, c_newparentdn, c_delete_p, sctrls, cctrls);
+  Check_LDAP_Result (ldapdata->err);
+
+  return self;
+}
+#endif
+
 /*
  * call-seq:
  * conn.delete(dn)  => self
@@ -1777,6 +1864,7 @@ Init_ldap_conn ()
                              rb_ldap_conn_s_allocate, 0);
 #endif
   rb_define_singleton_method (rb_cLDAP_Conn, "open", rb_ldap_conn_s_open, -1);
+  rb_define_singleton_method (rb_cLDAP_Conn, "open_uri", rb_ldap_conn_s_open_uri, 1);
   rb_define_singleton_method (rb_cLDAP_Conn, "set_option",
 			      rb_ldap_conn_s_set_option, 2);
   rb_define_singleton_method (rb_cLDAP_Conn, "get_option",
@@ -1794,6 +1882,9 @@ Init_ldap_conn ()
   rb_ldap_conn_define_method ("add", rb_ldap_conn_add_s, 2);
   rb_ldap_conn_define_method ("modify", rb_ldap_conn_modify_s, 2);
   rb_ldap_conn_define_method ("modrdn", rb_ldap_conn_modrdn_s, 3);
+#if defined(HAVE_LDAPCONTROL) && defined(HAVE_LDAP_RENAME_S)
+  rb_ldap_conn_define_method ("rename", rb_ldap_conn_rename_s, 6);
+#endif
   rb_ldap_conn_define_method ("delete", rb_ldap_conn_delete_s, 1);
 #if defined(HAVE_LDAP_COMPARE_S)
   rb_ldap_conn_define_method ("compare", rb_ldap_conn_compare_s, 3);
