@@ -13,7 +13,7 @@ class TC_LDAPTest < Test::Unit::TestCase
   # Get the LDAP host and base DN from /etc/ldap.conf.
   def setup
     unless @@conn && @@conn.bound?
-      File.open( '/etc/openldap/slapd.conf' ) do |f|
+      File.open( '/etc/ldap.conf' ) do |f|
         while line = f.gets
           if line =~ /^host\s+(\S+)$/
             @@host = $1
@@ -33,6 +33,6 @@ class TC_LDAPTest < Test::Unit::TestCase
     end
   end
 
-  #undef_method :default_test
+  undef_method :default_test
     
 end
